#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-torque-server-config v. 2.3.1
#
# Post-installation script for configuring the gLite torque server
# Guillermo Diez-Andino Sancho <Guillermo.Sancho@cern.ch>
#
# Version info: $Id: glite-torque-server-config.py,v 1.31.2.5 2005/11/25 22:21:57 dimeglio Exp $
#
# Usage: python glite-torque-server-config [-c|-v|-h|--start|--stop|--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --start         starts the pbs-torque services
#        --stop         stops the pbs-torque services
#        --status     shows the status of the pbs-torque services
#        --configure     configure the pbs-torque services
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

# import system libraries
import os,string
import sys
import getopt
import time

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                     # all config values from the XML file
global rgmaServicetool

class glite_torque_server:

    def __init__(self):
        self.verbose = 0
        self.version = "2.3.1"
        self.name = "glite-torque-server"
        self.friendly_name = "gLite Torque Server"
        #self.params = params
        self.glib = params['glib']
        params['module.version'] = self.version
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "          Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --start             starts all the pbs-torque services'
        print '    --stop               stops all the pbs-torque services'
        print '    --status            shows the status of the pbs-torque services'
        print '    --configure         configure the pbs-torque services'
        print '\n'

    def start(self):
        retCode = 0
        
        if self.stop() != 0:
            retCode = 1
        
        if self.glib.startProcess('maui','/etc/rc.d/init.d/maui start') != 0:
            retCode = 1
        if self.glib.startProcess('pbs_server','/etc/rc.d/init.d/pbs_server start') != 0:
            retCode = 1
            
        if params['use.log.parser'] == 'true':
            scriptParams = '%s/bin/BLParserPBS %s &' %(os.environ['GLITE_LOCATION'],params['pbs.log.parser.port'])
            self.glib.startProcess('BLParserPBS',scriptParams)
        
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if rgmaServicetool.start() != 0:
                retCode = 1
            
        return retCode
        
    def stop(self):
        retCode = 0
        
        if params['use.log.parser'] == 'true':
            scriptParams = '%s/bin/BLParserPBS' %os.environ['GLITE_LOCATION']
            self.glib.stopProcess(scriptParams,'')
            
        if self.glib.stopProcess('maui','/etc/rc.d/init.d/maui stop') != 0:
            retCode = 1
        if self.glib.stopProcess('pbs_server','/etc/rc.d/init.d/pbs_server stop') != 0:
            retCode = 1
        
        #-------------------------------------------------------------------
        # Stop the servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if rgmaServicetool.stop() != 0:
                retCode = 1
        
        return retCode
    
    def status(self):
        error_level = 0
        
        #-------------------------------------------------------------------
        # pbs_server
        #-------------------------------------------------------------------
        retval = os.system('/etc/rc.d/init.d/pbs_server status')
        if retval != 0:
            error_level = retval
         
        #-------------------------------------------------------------------
        # maui
        #-------------------------------------------------------------------
        retval = os.system('/etc/rc.d/init.d/maui status')
        if retval != 0:
            error_level = retval
        
        #-------------------------------------------------------------------------------
        # Servicetools
        #-------------------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if rgmaServicetool.status():
                error_level = 1
    
        return error_level    
    
    def backup(self,path):
        backupPath = "%s.1" % path
        if os.path.exists(backupPath):
            os.remove(backupPath)
        if os.path.exists(path):
            os.rename(path,backupPath)
            
        return 0

    def existsFile(self,path):
        if os.path.exists(path):
            print "File %s correctly created [\033[32m  OK  \x1b[0m]" %path
            return 0
        else:
            print "Problems to create file %s [\033[31mERROR\x1b[0m]" %path
            return 1        
    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------
    
    #Files involved: /etc/services /var/spool/pbs/server_name /var/spool/pbs/server_priv/nodes
    def configure(self):
        
        step = 0
        
        #------------------------------------------------------------------------
        #            STOPPING SERVICES
        #------------------------------------------------------------------------
        print "%s.- Stopping services " %step
        if self.stop() != 0:
            return 1
        step+=1
        #------------------------------------------------------------------------
        #            ADDING PORTS
        #------------------------------------------------------------------------
        print "%s.- Adding ports to /etc/services " %step
        file = open("/etc/services",'r')
        pbsTcp = 1
        pbsMomTcp = 1
        pbsReMomTcp = 1
        pbsReMomUdp = 1
        mauiTcp = 1
        for line in file.xreadlines():
            if line.find('pbs             15001/tcp') != -1:
                pbsTcp = 0
            if line.find('pbs_mom         15002/tcp') != -1:
                pbsMomTcp = 0
            if line.find('pbs_remom       15003/tcp') != -1:
                pbsReMomTcp = 0
            if line.find('pbs_remom       15003/udp') != -1:
                pbsReMomUdp = 0    
            if line.find('maui          15004/tcp') != -1:
                mauiTcp = 0    
        file.close()
        if pbsTcp == 0 and pbsMomTcp == 0 and pbsReMomTcp == 0 and pbsReMomUdp == 0 and mauiTcp == 0:
            print "The ports already existed in /etc/services [\033[32m  OK  \x1b[0m]"
        else:
            file = open('/etc/services','a')
            if pbsTcp == 1:
                file.write('\npbs             15001/tcp')
                print "pbs port added 15001/tcp [\033[32m  OK  \x1b[0m]"
            if pbsMomTcp == 1:    
                file.write('\npbs_mom         15002/tcp')
                print "pbs_mom port added 15002/tcp [\033[32m  OK  \x1b[0m]"
            if pbsReMomTcp == 1:
                file.write('\npbs_remom       15003/tcp')
                print "pbs_remom port added 15003/tcp [\033[32m  OK  \x1b[0m]"
            if pbsReMomUdp == 1:
                file.write('\npbs_remom       15003/udp')
                print "pbs_remom port added 15004/udp [\033[32m  OK  \x1b[0m]"
            if mauiTcp == 1:
                file.write('\nmaui          15004/tcp')
                print "maui port added 15004/tcp [\033[32m  OK  \x1b[0m]"
            file.close()
        step+=1
    
        #------------------------------------------------------------------------
        #            CREATING SERVER_NAME FILE
        #------------------------------------------------------------------------
        print "%s.- Adding server name /var/spool/pbs/server_name" %step
        self.backup("/var/spool/pbs/server_name")
        file = open('/var/spool/pbs/server_name','w')
        file.write("%s\n" % params['torque-server.name'])
        file.close()
        if self.existsFile('/var/spool/pbs/server_name') != 0:
            return 1
        step+=1
        
        #------------------------------------------------------------------------
        #            ADDING WORKER NODES
        #------------------------------------------------------------------------        
        print "%s.- Adding the worker nodes to /var/spool/pbs/server_priv/nodes" %step
        #To add new nodes it's necessary to stop the server (qterm -t quick command). No jobs will be lost
        #This file is read at start up time
        self.backup("/var/spool/pbs/server_priv/nodes")
        instances = self.glib.getInstances("wn-torque")
        file = open('/var/spool/pbs/server_priv/nodes','w')
        for instance in instances:
            try:
                iParams = self.glib.getInstanceParams(instance)
                file.write("%s np=%s %s\n" %(iParams['torque-wn.name'],iParams['torque-wn.number.processors'],iParams['torque-wn.attribute']))
            except ValueError, msg:
                print "Error reading torque-wn values, Cannot continue %s [\033[31mFAILED\x1b[0m]" % msg
                return 1
            except LookupError, msg:
                print "Error reading torque-wn values [\033[31mFAILED\x1b[0m]"
                return 1
        file.close()
        if self.existsFile('/var/spool/pbs/server_priv/nodes') != 0:
            return 1
        step+=1
        
        #------------------------------------------------------------------------
        #            CREATING PBS SERVER CONFIGURATION FILE
        #------------------------------------------------------------------------
        #If backup rollback (unset) and set with new configuration
        #Check default queue appears in the queues
        
        #tmpPath = "%s/torque-pbs-server.configuration" %os.environ['GLITE_LOCATION_LOG']
        serverParamsNew = {}
            
        print "%s.- Creating server configuration" %step
                 
        if params['torque-server.scheduling'] != "":
            serverParamsNew["scheduling"] = params['torque-server.scheduling']
            
        if params['torque-server.acl-host.enable'] != "":
            serverParamsNew["acl_host_enable"] = params['torque-server.acl-host.enable']
            if params['torque-server.acl-host.enable'] == "True":
                if params['torque-server.acl-host.list'] != "":
                    serverParamsNew['acl_hosts'] = params['torque-server.acl-host.list']
                else:
                    glib.printErrorMessage("The configuration variable acl_hosts can not be empty when torque-server.acl-host.list is set to True")
                    return 1        
            
        if params['torque-server.name'] != "":
            serverParamsNew["managers"] = 'root@%s' %params['torque-server.name']
            
        if params['torque-server.default.queue'] != "":
            serverParamsNew["default_queue"] = params['torque-server.default.queue']
            
        if params['torque-server.log.events'] != "":    
            serverParamsNew["log_events"] = params['torque-server.log.events']
            
        #if params['torque-server.query.other-jobs'] != "":    
        #    serverParamsNew["mail_from"] = "adm"
            
        if params['torque-server.query.other-jobs'] != "":
            serverParamsNew["query_other_jobs"] = params['torque-server.query.other-jobs']
            
        if params['torque-server.scheduler.interaction'] != "":    
            serverParamsNew["scheduler_iteration"] = params['torque-server.scheduler.interaction']
            
        if params['torque-server.default.node'] != "":    
            serverParamsNew["default_node"] = params['torque-server.default.node']
            
        if params['torque-server.node.pack'] != "":    
            serverParamsNew["node_pack"] = params['torque-server.node.pack']
            
        file.close()
        step+=1
        
                
        #Check PBS Server has been properly started if not start it
        
        #------------------------------------------------------------------------
        #            STARTING PBS_SERVER ...
        #------------------------------------------------------------------------
        print "%s.- Starting pbs_server" %step
        if self.glib.startProcess('pbs_server','/etc/rc.d/init.d/pbs_server start') != 0:
            return 1
        step+=1
                
        #------------------------------------------------------------------------
        #            MATCHING PBS SERVER CONFIGURATION
        #------------------------------------------------------------------------
        
        logServer = "%s/log-pbs-server" %os.environ['GLITE_LOCATION_LOG']
        print "%s.- Matching server configuration in file %s" %(step,logServer)
        serverParamsOld = {}

        if os.path.exists(logServer):
            file = open(logServer,"r")
            for line in file.xreadlines():
                key = line.split('=')[0]
                value = line.split('=')[1]
                value = value[0:len(value)-1]
                serverParamsOld[key] = value
            file.close()
            self.backup(logServer)
            
            file = open(logServer,"w")
            iterator = serverParamsOld.iterkeys()

            while True:
                try:
                    element = iterator.next()
                    
                    if serverParamsNew.has_key(element) == True:
                        if serverParamsNew[element] == serverParamsOld[element]:
                            file.write("%s=%s\n" %(element,serverParamsNew[element]))
                            del serverParamsNew[element]
                    else:
                        os.system("qmgr -c 'unset server %s" %element)
                        print "UNSETTING server parameter--> qmgr -c 'unset server %s' [\033[32m  OK  \x1b[0m]" %element
                except StopIteration:
                    break
            file.close()
            if self.existsFile(logServer) != 0:
                return 1
        step+=1
            
        #------------------------------------------------------------------------
        #            ESTABLISHING PBS SERVER CONFIGURATION
        #------------------------------------------------------------------------
        print "%s.- Establishing server configuration" %step    
        file = open(logServer,"a")
        iterator = serverParamsNew.iterkeys()

        if params['torque-server.acl-host.enable'] == "False":
            os.system("qmgr -c 'unset server acl_hosts'")
                    
        while True:
            try:
                element = iterator.next()
                file.write("%s=%s\n" %(element,serverParamsNew[element]))
                os.system("qmgr -c \"set server %s = '%s'\"" %(element,serverParamsNew[element]))
                print "SETTING server parameter--> qmgr -c \"set server %s = '%s'\" [\033[32m  OK  \x1b[0m]" %(element,serverParamsNew[element])
            except StopIteration:
                break
        file.close()
        step+=1

        #Check the server is running
        #It's considered that only one pbs_server is running
        #/usr/sbin/pbs_server
        #os.system("/usr/bin/qmgr < %s" %tmpPath)
        
            
        #------------------------------------------------------------------------
        #            CREATING THE PBS QUEUES CONFIGURATION FILE
        #------------------------------------------------------------------------
        print "%s.- Creating the queues configuration " %step
        
        queuesNew = {}
        queuesOld = {}
        
        instances = self.glib.getInstances("pbs-queue")
                
        for instance in instances:
            iParams = self.glib.getInstanceParams(instance)
            queuesNew[iParams['queue.name']] = iParams['queue.name']
        
        queuesList = "%s/queuesList" %os.environ['GLITE_LOCATION_LOG']
                
        if os.path.exists(queuesList):
            file = open(queuesList,"r")
            for line in file.xreadlines():
                key = line[0:len(line) -1]
                queuesOld[key] = key
            file.close()
            self.backup(queuesList)
            
            file = open(queuesList,"w")
            iterator = queuesOld.iterkeys()
            while True:
                try:
                    element = iterator.next()
                    if queuesNew.has_key(element) == False:
                        if params['torque-server.force'] == "True":
                            os.system("qmgr -c 'delete queue %s'" %element)
                            print "DELETING QUEUE %s-->qmgr -c 'delete queue %s'" %(element,element)
                        else:
                            print "The queue %s does not appear in the configuration file anymore. It won't be removed [\033[32m  WARNING  \x1b[0m]" %element
                            file.write("%s\n" %element)
                except StopIteration:
                    break
            file.close()

        step+=1
        
        #------------------------------------------------------------------------
        #            RUNNING PBS QUEUE CONFIGURATION
        #------------------------------------------------------------------------
        instances = self.glib.getInstances("pbs-queue")

        iterator = queuesNew.iterkeys()
        file = open(queuesList,"a")
        while True:
            try:
                element = iterator.next()
                for instance in instances:
                    iParams = self.glib.getInstanceParams(instance)
                    if iParams['queue.name'].find(element) == 0:
                        break
        
                if queuesOld.has_key(element) == False:
                    os.system("qmgr -c 'create queue %s'" %element)
                    print "CREATING QUEUE %s-->qmgr -c 'create queue %s'" %(element,element)
                
                os.system("qmgr -c 'set queue %s queue_type = %s'" %(iParams['queue.name'],iParams['queue.type']))        
                os.system("qmgr -c 'set queue %s resources_max.cput = %s'" %(iParams['queue.name'],iParams['queue.resources.max.cpu.time']))
                os.system("qmgr -c 'set queue %s resources_max.walltime = %s'" %(iParams['queue.name'],iParams['queue.max.wall.time']))
                os.system("qmgr -c 'set queue %s enabled = %s'" %(iParams['queue.name'],iParams['queue.enabled']))
                os.system("qmgr -c 'set queue %s started = %s'" %(iParams['queue.name'],iParams['queue.started']))
                os.system("qmgr -c 'set queue %s acl_group_enable = %s'" %(iParams['queue.name'],iParams['queue.acl.group.enable']))
                                
                if iParams['queue.acl.group.enable'] == "True":
                    os.system("qmgr -c \"set queue %s acl_groups = '%s'\"" %(iParams['queue.name'],iParams['queue.acl.groups']))
                else:
                    os.system("qmgr -c \"unset queue %s acl_groups\"" %(iParams['queue.name']))
                    
                print "Parameters set up to queue %s [\033[32m  OK  \x1b[0m]" %element
                file.write("%s\n" %element)        
            except StopIteration:
                break
        file.close()
        step+=1    
        
        #print "%s.- Establishing queues configuration" %step
        #file = os.popen('/usr/sbin/pbs_server -t create','w',0)
        #file.write('y')
        #file.close()
        
        #------------------------------------------------------------------------
        #            CREATING /opt/edg/etc/edg-pbs-shostsequiv.conf FILE
        #------------------------------------------------------------------------
        print "%s.- Creating file /opt/edg/etc/edg-pbs-shostsequiv.conf" %step
        self.backup("/opt/edg/etc/edg-pbs-shostsequiv.conf")
        file = open("/opt/edg/etc/edg-pbs-shostsequiv.conf","w")
        file.write("SHOSTSEQUIV = /etc/ssh/shosts.equiv\n")
        file.write("NODES =\n")
        file.write("PBSBIN      = /usr/bin\n")
        file.close()
        if self.existsFile('/opt/edg/etc/edg-pbs-shostsequiv.conf') != 0:
            return 1
        step+=1
        
        #------------------------------------------------------------------------
        #            ADDING edg-pbs-shostequiv.conf to /etc/cron.d directory
        #------------------------------------------------------------------------
        print "%s.- Adding edg-pbs-shostequiv.conf to /etc/cron.d directory" %step
        file = open("/etc/cron.d/edg-pbs-shostsequiv","w")
        file.write("05 1,7,13,19 * * * root /opt/edg/sbin/edg-pbs-shostsequiv\n")
        file.close()
        if self.existsFile('/etc/cron.d/edg-pbs-shostsequiv') != 0:
            return 1
        os.system("chmod +x /etc/cron.d/edg-pbs-shostsequiv")
        step+=1
        
        #------------------------------------------------------------------------
        #            RUNNING edg-pbs-shostsequiv
        #------------------------------------------------------------------------
        print "%s.- Running edg-pbs-shostsequiv " %step
        os.system("/opt/edg/sbin/edg-pbs-shostsequiv")
        step+=1    
        
        #------------------------------------------------------------------------
        #                       DUPLICATED KEY ENTRIES
        #------------------------------------------------------------------------
        
        print "%s.Looking for duplicated entries" %step
        scriptFile = "%s/searchKeys.sh" %os.environ['GLITE_LOCATION_TMP']
        self.backup(scriptFile)
        file = open(scriptFile,"w")
                                                                                                                                                   
        file.write("#!/bin/bash\n")
        file.write("for hostname in %s; do\n" %(params['torque-server.name']))
        file.write("    if [ -f /etc/ssh/ssh_known_hosts ];then \n")
        file.write("            grep -v $hostname /etc/ssh/ssh_known_hosts  > /etc/ssh/ssh_known_hosts.tmp\n")
        file.write("            /usr/bin/ssh-keyscan -t rsa $hostname >> /etc/ssh/ssh_known_hosts.tmp 2>/dev/null\n")
        file.write("            if [ $? = 0 ]; then\n")
        file.write("                    mv -f /etc/ssh/ssh_known_hosts.tmp  /etc/ssh/ssh_known_hosts\n")
        file.write("            fi\n")
        file.write("    fi\n")
        file.write("done\n")
        file.close()
        os.system("sh %s" %scriptFile)
        step +=1
        
        #------------------------------------------------------------------------
        #            CREATING CONFIGURATION FILE /opt/edg/etc/edg-pbs-knownhosts.conf
        #------------------------------------------------------------------------
        print "%s.- Creating file /opt/edg/etc/edg-pbs-knownhosts.conf" %step
        self.backup("/opt/edg/etc/edg-pbs-knownhosts.conf")
        file = open("/opt/edg/etc/edg-pbs-knownhosts.conf","w")
        file.write("NODES = %s\n" %(params['torque-server.name']))
        file.write("PBSBIN = /usr/bin\n")
        file.write("KEYTYPES = rsa1,rsa,dsa\n")
        file.write("KNOWNHOSTS = /etc/ssh/ssh_known_hosts\n")
        file.close()
        if self.existsFile('/opt/edg/etc/edg-pbs-knownhosts.conf') != 0:
            return 1
        step+=1
        
        #print "%s.- Looking for duplicated keys " %step
        #step+=1
                
        #------------------------------------------------------------------------
        #            ADDING /opt/edg/etc/edg-pbs-knownhosts.conf TO CRON.D DIRECTORY
        #------------------------------------------------------------------------
        print "%s.- Adding edg-pbs-knownhosts to the cron.d directory" %step
        file = open("/etc/cron.d/edg-pbs-knownhosts","w")
        file.write("03 1,7,13,19 * * * root /opt/edg/sbin/edg-pbs-knownhosts\n\n")
        file.close()
        if self.existsFile('/etc/cron.d/edg-pbs-knownhosts') != 0:
            return 1
        os.system("chmod +x /etc/cron.d/edg-pbs-knownhosts")
        step+=1
        
        #------------------------------------------------------------------------
        #            RUNNING /opt/edg/sbin/edg-pbs-knownhosts
        #------------------------------------------------------------------------
        print "%s.- Running  /opt/edg/sbin/edg-pbs-knownhosts " %step
        os.system("/opt/edg/sbin/edg-pbs-knownhosts")
        step+=1
        
        #------------------------------------------------------------------------
        #            CREATING THE SSHD CONFIGURATION
        #------------------------------------------------------------------------
        hostBasedAuth = 1
        ignoreUserKnownHost = 1
        ignoreRHosts = 1
        print "%s.- Changing /etc/ssh/sshd_config" %step
        file = open("/etc/ssh/sshd_config",'r')
        for line in file.xreadlines():
            if line.find('HostbasedAuthentication yes') == 0:
                hostBasedAuth = 0
            if line.find('IgnoreUserKnownHosts yes') == 0:
                ignoreUserKnownHost = 0    
            if line.find('IgnoreRhosts yes') == 0:
                ignoreRHosts = 0    
        file.close()
                
        if hostBasedAuth == 0 and ignoreUserKnownHost == 0 and ignoreRHosts == 0:
            print "sshd_config already configured [\033[32m  OK  \x1b[0m]"
        else:    
            file = open('/etc/ssh/sshd_config','a')
            if hostBasedAuth == 1:
                file.write("\nHostbasedAuthentication yes")
                print "Added HostbasedAuthentication yes to /etc/ssh/sshd_config [\033[32m  OK  \x1b[0m]" 
            if     ignoreUserKnownHost == 1:
                file.write("\nIgnoreUserKnownHosts yes")
                print "Added IgnoreUserKnownHosts yes to /etc/ssh/sshd_config [\033[32m  OK  \x1b[0m]"
            if     ignoreRHosts == 1:
                file.write("\nIgnoreRhosts yes")
                print "Added IgnoreRhosts yes to /etc/ssh/sshd_config [\033[32m  OK  \x1b[0m]"
            file.close()
            print "sshd_config changed [\033[32m  OK  \x1b[0m]"
            
        if self.existsFile('/etc/ssh/sshd_config') != 0:
            return 1
        step+=1
        
        #------------------------------------------------------------------------
        #            RESTARTING SSHD
        #------------------------------------------------------------------------
        print "%s.- Restarting sshd " %step
        os.system("/etc/rc.d/init.d/sshd restart") 
        step+=1
        
        #------------------------------------------------------------------------
        #            STOPPING THE PBS SERVER
        #------------------------------------------------------------------------
        print "%s.- Stopping the pbs server to apply changes" %step
        os.system('/etc/rc.d/init.d/pbs_server stop')            
        step+=1
            
        #------------------------------------------------------------------------
        #            CREATING MAUI CONFIGURATION
        #------------------------------------------------------------------------        
        print "%s.- Creating maui configuration file in /var/spool/maui/maui.cfg" %step
        self.backup("/var/spool/maui/maui.cfg")
        file = open("/var/spool/maui/maui.cfg","w")
        file.write("SERVERHOST              %s\n" %params['torque-server.name'])
        file.write("ADMIN1                  root\n")
        file.write("ADMINHOST               %s\n" %params['torque-server.name'])
        file.write("RMCFG[base]             TYPE=PBS\n")
        file.write("SERVERPORT              %s\n" %params['maui.server.port'])
        file.write("SERVERMODE              %s\n" %params['maui.server.mode'])
        file.write("RMPOLLINTERVAL          %s\n" %params['maui.rm.poll.interval'])
        file.write("LOGFILE                 %s\n" %params['maui.log.filename'])
        file.write("LOGFILEMAXSIZE          %s\n" %params['maui.log.max.size'])
        file.write("LOGLEVEL                %s\n" %params['maui.log.level'])
        file.write("DEFERTIME                %s\n" %params['maui.defer.time'])
        file.close()
        if self.existsFile('/var/spool/maui/maui.cfg') != 0:
            return 1
        step+=1
        
        #------------------------------------------------------------------------
        #            RESTARTING MAUI 
        #------------------------------------------------------------------------        
        #print "%s.- Starting maui" %step
        os.system("/sbin/chkconfig maui on")
        #os.system("/etc/rc.d/init.d/maui start")
        #step+=1
        
        #-------------------------------------------------------------------
        # R-GMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if self.verbose:
                print "%s - Configuring the R-GMA Service Publisher... " % step
    
            # Instantiate the rgma-servicetool class
            rgmaServicetool = gliteRgmaServicetool()
            rgmaServicetool.verbose = self.verbose
            
            # Create PBS instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['pbs.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['pbs.serviceName'])
            servicetoolInstance.setServiceType(params['pbs.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['pbs.statusScript'])
            servicetoolInstance.setEndpoint(params['pbs.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create Maui instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['maui.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['maui.serviceName'])
            servicetoolInstance.setServiceType(params['maui.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['maui.statusScript'])
            servicetoolInstance.setEndpoint(params['maui.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
            print "%s - Configuring the R-GMA Service Publisher                      " % step,
            glib.printOkMessage()
            step += 1
        
        return 0
        
        #print "%s.- Configuring the servicetool" %step
        #if rgmaServicetool.configure(self.glib) == 0:
        #    print "FAILED configuring the service tool"
        #    return 1
        #print "OK Configuring the Service Tool"
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)
        #glib.export('GLOBUS_LOCATION')
    glib.export('PBS_SPOOL_DIR',params['PBS_SPOOL_DIR'])
        
    # bin and lib paths

    #glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/lib" % (os.environ['GLOBUS_LOCATION'],os.environ['GLITE_LOCATION']))
    #/sbin/ldconfig
        # Set environment
    
    #Adding JAVA_PATH     glib.addEnvPath("PATH","%s/bin" % os.environ['JAVA_HOME]) after PATH
    glib.setUserEnv()    
    return 0
    
#-------------------------------------------------------------------------------
# Init params
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    
    params['pbs.serviceName'] = 'Torque PBS Server service at %s' % glib.fq_hostname
    params['pbs.serviceType'] = 'pbs.torque.server'
    params['pbs.statusScript'] = '/etc/rc.d/init.d/pbs_server status'
    params['pbs.endpoint'] = 'not available'
    
    params['maui.serviceName'] = 'Maui Scheduler service at %s' % glib.fq_hostname
    params['maui.serviceType'] = 'pbs.torque.maui'
    params['maui.statusScript'] = '/etc/rc.d/init.d/maui status'
    params['maui.endpoint'] = 'not available'
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '[FAILED] This script must be run as root'
        sys.exit(1)
    
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    loadDefaults(params)
    params['glib'] = glib
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
        
    if glib.loadConfiguration(["%s/../glite-torque-server.cfg.xml" % glib.getScriptPath(), "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()], params):
        print "An error occurred while configuring the service"
        sys.exit(1)
        
    # set verbosity
    if params.has_key('glite.installer.verbose'):
        if (params['glite.installer.verbose'] == "true"):
            verbose=1
        else:
            verbose=0
    else:
        verbose = 0
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
    
    service = glite_torque_server()
    service.verbose = verbose
        
    rgmaServicetool = gliteRgmaServicetool()     # Instantiate the rgma servicetool class
    rgmaServicetool.verbose = verbose
        
    path = '%s/etc' % os.environ['GLITE_LOCATION']
    
    if not os.path.exists(path):
        print '[FAILED] %s/etc does not exist! Please verify your gLite installation' % os.environ['GLITE_LOCATION']
        sys.exit (1)
    else:
        print '[OK] %s/etc exists' % os.environ['GLITE_LOCATION']

    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','configure','stop','start','status', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "[FAILED] Unknown options(s)")
        sys.exit(1)

    # Check cli options
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        if o == "--start":
            if service.start() == 0:
                print "[OK] The %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "[FAILED] An error occurred while starting the %s service" % service.friendly_name
                sys.exit(1)
        if o == "--stop":
            if service.stop() == 0:
                print "[OK] The %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "[FAILED] An error occurred while stopping the %s service" % service.friendly_name
                sys.exit(1)
        if o == "--configure":
            # Configure the service
            if service.configure() == 0:
                glib.registerService()
                print "\n%s configuration successfully completed     " % service.friendly_name,
                glib.printOkMessage()
                sys.exit(0)
            else:
                print "\nAn unrecoverable error occurred while configuring the %s service    " % service.friendly_name,
                glib.printFailedMessage()
                sys.exit(1)
                
        if o == "--status":
            retval = service.status()
            sys.exit(retval)            
 
    # Print configuration parameters
    if verbose:
        glib.print_params(params)
        service.copyright()
        service.showVersion()
        service.banner()
        
    service.usage()
